;;
;; ** Thu Sep 07 20:08:37 2006 **
;;
;; button.asm - abstracts the details of buttons for the game.  Generates
;;	a "button press" for the game and deals with debounce as well as
;;	allowing the game to track consumption of a button press.
;;
#include "portdef.h"
#include "regdef.h"

;; Buttons are pressed, and the action takes place upon the downstroke.
;; However, any bouncing (up/down within a short time) is filtered out.
;; The consumer (the game) acknowledges the button press when it wants
;; to.  However, another event isn't generated until the button moves
;; again.
;;
;; The way this works is pretty simple.  A button LAST_STATE register
;; indicates the state of the button during the last loop.  If the
;; state in the current loop is the same as the state in the last loop,
;; then nothing happens.  If the two are different, then it represents
;; a "down" or "up" event for the button.
	
#include "button.mac"
#include "random.h"
#include "random.mac"

;;;
;;; This routine exports the following symbols
;;; 
   global	ButtonProcess		; called once per loop, checks buttons
   global	ButtonInit		; called to initialize buttons

PROG1	CODE

;;; =======================================================================
;;; =======================================================================


;;
;; ButtonProcess - processes the current state of the buttons.  Should
;;		   normally be called at the top of a loop.  It will set
;;		   the button state appropriately.  It should only be
;;		   called once in a loop.  Uses WORK0
;;
;;    KEY DEBOUNCE - buttons are "debounced" by ensuring that a state
;;		is maintained for a short time before changing state.
;;;
;;;   HARDWARE-WISE - a button is "pressed" when it goes LOW, otherwise
;;;		a resistor pulls it to HIGH when the button is NOT pressed.
;;
ButtonProcess0:
		ButtonSample		0
		ButtonSkipOnChange	0
		goto			ButtonReset0
		ButtonCheckBounce	0
		return
		ButtonToggle		0
		RandomSeed
ButtonReset0:
		ButtonResetBounce	0
		return

ButtonProcess1:	
		ButtonSample		1
		ButtonSkipOnChange	1
		goto			ButtonReset1
		ButtonCheckBounce	1
		return
		ButtonToggle		1
		RandomSeed
ButtonReset1:
		ButtonResetBounce	1
		return

ButtonProcess:	
	call	ButtonProcess0
	call	ButtonProcess1
	return

;;
;; ButtonInit - Initialize Button Stuff.  All Buttons Start Out As Not
;;		Pressed.  So If A Button Is Down When Starting Up, It
;;		Will Generate A Press Immediately.  Both Button State
;;		Registers Start Off At Zero, Which Indicates That The
;;		Button Was Last Seen As "Off".  "On" Refers To A Button
;;		In The "Pressed" State.
;;
;;
ButtonInit:
	movlw	BUTTON_RELEASE
	movwf	BUTTON_STATE_0
	movwf	BUTTON_STATE_1
	movwf	BUTTON_LAST_STATE_0
	movwf	BUTTON_LAST_STATE_1

	ButtonResetBounce	0
	ButtonResetBounce	1

	return

;;; =======================================================================
;;; =======================================================================

	END
	
