;;
;; ** Thu Sep 07 19:56:40 2006 **
;;
;; display.asm - implements the display routines.  Includes output generation
;;   as well as the interface to the display by all of the other routines.
;;

#include "portdef.h"
#include "regdef.h"

;;
;; although the COLUMNS is defined here to be five, AND the following
;; code obeys this setting, the code isn't written to have more than
;; 8 columns, further, regdef.h is static at 5 columns currently

#define COLUMNS	5

	global	DisplayInit	; initializes the display
	global	DisplayUpdate	; called in the main loop to update the display
	global	SetBit		; sets a given (x,y) bit
	global	ClearBit	; clears a given (x,y) bit - resets blink propery too
	global	CheckBit	; checks a given (x,y) bit - Z is set if zero (not set)
	global	BlinkSetting	; sets the blink property (0-3) for the next bits
	global	ToggleBit	; togges a given (x,y) bit
	global	DisplayClear	; clears the display - although it takes COLUMNS loops
				;   to update the LEDS themselves
	global	ColIndexSet	; external programs sometimes directly load the display
				;  though should ALWAYS use the macro interface
	global	DisplayRotate	; used to load the display from a rotated (top-down) display
	global	DisplayTempBlank ; temporarily blanks the display
	global	CreateMask
	
PROG1	CODE

;;
;; ColIndexSet - given W to be the target column, point the
;;		 index register at the appropriate column data word.
;;
ColIndexSet:
		movwf	FSR
		addwf	FSR
		addwf	FSR		;; 3 * W added to the base
		movlw	COLDATA_BASE	;; W now has register index
		addwf	FSR		;; INDF points to =  COLDATA_BASE + 3*W 
		return

;;
;; DisplayUpdate - called in the main loop, this routine exports the current
;;	display to the display hardware, one column at a time.  In other words,
;;	one column is output to the display at each call.  At the end of the
;;	call, the column is incremented so that the next call will display the
;;	next column.

DisplayUpdate:

	;; there are a set of registers which contain the display
	;;
	;;    ,-----------,
	;;    | O O O O O | 6
	;;    | O O O O O | 5   Display
	;;    | O O O O O | 4   5 columns and 7 rows
	;;    | O O O O O | 3
	;;    | O O O O O | 2
	;;    | O O O O O | 1
	;;    | O O O O O | 0
	;;    '-----------'
	;;	0 1 2 3 4
	;;  The row data on each column (7 bits) is latched and
	;;  column is enabled separately.  Resistors on the row data (x7).
	;;  Up to 7 LEDs on at once.
	;;	
	;;  During each display update one of the columns is turned on.
	;;  So given a 18 ms loop (55 Hz), and 5 columns, each column is 
	;;  is on for 11 Hz or (* 11 18) 198 ms/sec - should work just peachy
	;;

	call	BlinkerUpdate
	call	ExportDisplay
	call	NextColumn

	return

;;
;; NextColumn - moves the CURCOL indicator to the "next" column.
;;	        Note that columns move from high to low.
;;

NextColumn:
	movf	CURCOL,W		;; check for zero of current column (and load W)
	skpnz
	movlw	COLUMNS			;; if we were at zero, reload W to high column
	movwf	CURCOL			;; potentially updated current column
	decf	CURCOL			;; decremented appropriately
	return

;;
;; ExportDisplay - given the current configuration, move the internal concept of the
;;		   display state to the LEDs.  The CURRENT column is displayed, and
;;		   then is advanced to the next column at the end of the display
;;
ExportDisplay:
	call	LatchColumnData		;; latch in the column data (side-effect of blanking display)
	call	SetColumnSelector	;; and turn on the appropriate column
	call	UnBlankDisplay		;; turn display back on
	return

;;
;; UnBlankDisplay - turn display back on.
;;
UnBlankDisplay:	
	bcf	DISPLAY_LATCH
	return

;;
;; LatchColumnData - latch-in the data for an entire column.  CURCOL is used to select
;;		     the appropriate data.
;;

LatchColumnData:
	movf	CURCOL,W
	call	ColIndexSet		;; sets the Index register to point at target column data
	movf	INDF,W			;; get target column data into W
	call	BlinkProcess		;; change it for blinking if necessary
	movwf	DISPLAY_DATA		;; multi-plexed DATAOUTPUT lines
	bsf	DISPLAY_LATCH		;; latch in data, and blank display
	return

;;
;; SetColumnSelector - sets the lines for selecting the right column.
;;		       Column selection is a one-hot, so it needs to
;;		       be shifted into the appropriate output register.
;;		       CURCOL is used as the column to select.

SetColumnSelector:
	movf	CURCOL,W
	call	CreateMask	;; get a mask for the CURCOL
	xorlw	0xff		;; new board design needs inverted COL output lines
	movwf	DISPLAY_DATA	;; multi-plexed DATAOUTPUT lines
	return
		
;;
;; DisplayTempBlank() - temporarily blanks the display.  Used for things like
;;			playing music, which takes over the processor not letting
;;			the display update appropriately.
DisplayTempBlank:
	movlw	0xff
	movwf	DISPLAY_DATA	;; new board design needs inverted COL output lines
;;; 	clrf	DISPLAY_DATA	;; old board needed simple clear of COL output lines
	return

;;
;; DisplayInit - initializes display to a known state.  Note that it is blank
;;		 when initialized.
;;

DisplayInit:
	;; initial state of CURCOL is the high column

	movlw	COLUMNS
	movwf	CURCOL
	decf	CURCOL		;; set current column to the high column

	;; blank the display (latches in random data)

	bsf	DISPLAY_LATCH

	;; clear out the display memory

	call	DisplayClear

	;; set up the blinker

	movlw	1
	movwf	BLINK_COUNTER
	clrf	BLINKER

	clrf	BLINK_SETTING

	return

DisplayClear:
	movlw	COLUMNS
	movwf	WORK0

DisplayClearLoop:	
	movf	CURCOL,W
	call	ColIndexSet
	movlw	0x7f		; upper bit doesn't isn't an output anyway
	movwf	INDF
	incf	FSR
	clrf	INDF		; clear low blink bits
	incf	FSR
	clrf	INDF		; clear high blink bits
	call	NextColumn
	decfsz	WORK0
	goto	DisplayClearLoop

	return

;;
;; BlinkerUpdate() - updates all blinker stuff
;;
BlinkerUpdate:
	decfsz	BLINK_COUNTER
	return

	movlw	BLINK_RATE
	movwf	BLINK_COUNTER

	incf	BLINKER
	return	
	
;; 
;; SelectBit - (used by SetBit and ClearBit)
;;	       Sets up the conditions for setting or clear bits in the display.
;;	       Input is W containing the (x,y) coordinate of the target bit.
;;	       (x) is the column selector, and (y) is the column data.
;;	       4-bits for each within W.
;;	       Upon output, the INDF(FSR) is set to the right data location
;;	       and W contains a bit mask for the target bit.
;;	       WORK0 is used

SelectBit:
	movwf	WORK0

	swapf	WORK0,W		;; get column selector in to W
	andlw	0x0f		;; but need to clear off other bits

	call	ColIndexSet	;; after this, INDF(FSR) points to data bits

	movf	WORK0,W
	andlw	0x0f		;; now work with (y) bits

	;; INDF(FSR) is now set right

	;; tail return is done for the next routine to save a stack slot
	;; note that this routine sets W to the bitmask

	goto	CreateMask	;; given y, create a mask for the right bit

;;
;; SetBit/ClearBit/ToggleBit - set/clear/toggle the given display bit.  W has the target bit.
;;	    Bits are given in hex where the upper 4 bits are the column selector (x)
;;	    and the lower 4 bits are the row selector (y).  Note that the state for the
;;;	    row bits is inverted, a zero is "on".
;;;
;;; If (x,y) is out of bounds, currently bad things happen.

ClearBit:
	call	SelectBit
	iorwf	INDF		;; this will clear the target bit in the target column (sets it to 1)

	;; cleear the blink bits too

	xorlw	0xff		;; invert W for clearing blink bits

	incf	FSR		;; point to low blink bits
	andwf	INDF

	incf	FSR		;; point to high blink bits
	andwf	INDF

	return

SetBit:
	call	SelectBit
	xorlw	0xff		;; has the effect of inverting W
	andwf	INDF		;; sets the target bit in the target column (sets it to 0)

	xorlw	0xff		;; invert W back again

	incf	FSR		;; point to low blink bits
	btfsc	BLINK_SETTING,0	
	iorwf	INDF		;; set the low blink bits if needed

	incf	FSR		;; point to high blink bits
	btfsc	BLINK_SETTING,1	
	iorwf	INDF		;; set the high blink bits if needed

	return

ToggleBit:
	call	SelectBit
	xorwf	INDF		;; has the effect of toggling the given bit
	return			;; NOTE that blinking isn't affected

;;
;; CheckBit() - sets the Z flag if the given bit IS set.  No Z flag means it is not set.
;;		Incoming - W has the bit (x,y) to check.  Note that the blink status
;;		doesn't affect the outcome at all.  Remember that data is inverted.
;;
CheckBit:
	call	SelectBit
	andwf	INDF,W		; simply check the mask against the bit
	return			; if this is zero, then the bit was not set.

BlinkSetting:
	movwf	BLINK_SETTING
	return

;;
;; BlinkProcess() - given the current bits that would otherwise be sent to the display,
;;			potential "blink" them.  That is, the bits that are meant to be
;;			on get turned off if they are being "blinked" and the current
;;			timing indicates that the state is off.
;;
;;			If both high and low blink bits are on, you get a "intermittent
;;			fast blink" (for lack of a better term).
;;
;;		Input: W has display data (inverted - 0 is on), FSR/INDF points to display data
;;		Output: W has display data (also inverted)
;;		Uses: WORK0
;;
;;	NOTE - blink data is NOT inverted (1 means "blink")
;;
;;	NOTE - prototype board has 2/0 for slow/fast blink
;;		production board has 3/1.
;;
BlinkProcess:
	movwf	WORK0

	;; check the slow bits first

	movlw	0xff
	btfss	BLINKER,2	;; slow blinking on this bit
	movlw	0x00

	incf	FSR		;; point INDF to low blink bits
	andwf	INDF,W		;; W now has "slow" blink state, which can turn OFF display bits if 1
	iorwf	WORK0

	;; now fast bits

	movlw	0xff
	btfss	BLINKER,1	;; fast blinking on this bit
	movlw	0x00

	incf	FSR		;; point INDF to high blink bits
	andwf	INDF,W		;; W now has "medium" blink state, which can turn OFF display bits if 1
	iorwf	WORK0,W

	return

;;
;; CreateMask - given a number (less that 7!) create a bitmask for it.
;;		The bitmask starts at bit zero and can go up to bit seven.
;;		W has the number coming in, and the mask coming out.
;;		Result is (somewhat) undefined if the incoming W is greater
;;		than 7.
;;		Uses WORK1 and WORK2.
;;

CreateMask:
	movwf	WORK1		;; store target number
	incf	WORK1		; adjust a bit to make loop below work

	bsf	STATUS,C
	clrf	WORK2		; WORK2 is the mask accumulator

CreateMaskLoop:

	rlf	WORK2		;; move the bit over (first time shifts in C)
	decfsz	WORK1
	goto	CreateMaskLoop

	movf	WORK2,W
	return
	

;;
;; DisplayRotate() - given a rotated display, convert it to the real display
;;			(and push it out to the display)
;;			W has the register number of the start of the display,
;;			which goes from top to bottom
;;		   6	o o o o o  reg0
;;		   5	o o o o o  reg1       So the bits in column 0 are made
;;		   4	o o o o o  reg2       from each bit 4 of the 7 registers.
;;		   3	o o o o o  reg3
;;		   2	o o o o o  reg4
;;		   1	o o o o o  reg5
;;		row0	o o o o o  reg6
;;                      ---------
;;               bit    4 3 2 1 0
;;                      ---------
;;		 col	0 1 2 3 4
;;
;; 			Uses ARG0, ARG1, and WORK0, WORK1
;; 
DisplayRotate:
	movwf	ARG1		; ARG1 keeps the rotated display first register

	call	DisplayClear	; just to ensure blinking bits are cleared
	
	;; used to simply bit-shift ARG0, but since the subroutine trashes
	;; the carry bit, moved to just brute-force for each bit

	clrf	ARG0		; used as the mask for the rotated data bits
	bsf	ARG0,0
	call	DisplayRotateSub ; get bit 0 of all rotated registers (in ARG1)
	movwf	COLDATA_4


	clrf	ARG0		; used as the mask for the rotated data bits
	bsf	ARG0,1
	call	DisplayRotateSub ; get bit 1 of all rotated registers (in ARG1)
	movwf	COLDATA_3
	rlf	ARG0

	clrf	ARG0		; used as the mask for the rotated data bits
	bsf	ARG0,2
	call	DisplayRotateSub ; get bit 2 of all rotated registers (in ARG1)
	movwf	COLDATA_2
	rlf	ARG0

	clrf	ARG0		; used as the mask for the rotated data bits
	bsf	ARG0,3
	call	DisplayRotateSub ; get bit 3 of all rotated registers (in ARG1)
	movwf	COLDATA_1
	rlf	ARG0

	clrf	ARG0		; used as the mask for the rotated data bits
	bsf	ARG0,4
	call	DisplayRotateSub ; get bit 4 of all rotated registers (in ARG1)
	movwf	COLDATA_0

	return

;;
;; DisplayRotateSub() - subroutine called by DisplayRotate() which will compose
;;			a single column based upon the data in the rotated regs.
;;			ARG1 has the location of the first register
;;			ARG0 has a mask for getting the right bit.
;;			W comes back with the column data.
;;			Uses WORK0 and WORK1.
;;;			Uses INDF/FSR.
;;
DisplayRotateSub:
	movf	ARG1,W
	movwf	FSR		; point to the first of the 7 registers
	clrf	WORK0		; the accumulator for the results
	bcf	STATUS,C

	movlw	7		; seven loops to get all data
	movwf	WORK1

DRSLoop:
	movf	INDF,W		; get data into W
	andwf	ARG0,W		; and combine it with the mask
	skpz
	bsf	STATUS,C	; if it wasn't zero then set the target bit
	rlf	WORK0		; shift it in
	incf	FSR		; next register
	decfsz	WORK1
	goto	DRSLoop

	movf	WORK0,W		; return the accumulated data in W
	return
	
;;; =======================================================================
;;; =======================================================================

	END
