;;
;; music.asm
;;   Used to play actual "music" - to the best of the speaker's ability.
;;   Unlike "sound.asm" these routines don't use the standard display/update
;;   loop, and instead play straight music...not allowing other things to
;;   occur in the mean-time.
;;
;;   The user creates a nice little table of notes to play with "retlw" instructions
;;	through the MusicNoteTable macros.
;;
#include "regdef.h"
#include "portdef.h"
#include "utility.mac"
#include "random.h"
#include "display.h"

	global	MusicTimerInit
	global	MusicPlayNote
	global	MusicScorePlayCall

PROG1	CODE

;;;
;;; MusicTimerInit() - initialize the timer for use in playing music.  This thing
;;;			sets up the prescaler to maximum.  Then checking the timer
;;;			for overflow.
;;;
MusicTimerInit:
	bcf	INTCON,T0IE	; clear timer interrupt (though GIE should be off anyway)
	
	BankSelect	1

	bcf	OPTION_REG,T0CS	; timer on internal clock
	bcf	OPTION_REG,PSA	; assign the pre-scaler to the TIMER0
	bcf	OPTION_REG,PS0	; set the three pre-scaler bits for a 1:128 TIMER0 prescaler
	bsf	OPTION_REG,PS1
	bsf	OPTION_REG,PS2

	BankSelect	0

	return

;;;
;;; MusicScorePlayCall() - Plays the given musical score.  Normally called from
;;;				a macro of the "same" name.
;;;
;;;				A music "score" is a table (with retlw's) of notes
;;;				along with durations.  Notes are defined with the
;;;				MusicNoteTable macros.
;;;
;;;				Note that this thing "takes over" the processor in
;;;				that it doesn't let other things run in the mean-time
;;;				including display processing.
;;;
;;;				Uses ARG0 as pointer into the table.
;;;

MusicScorePlayCall:
	call	MusicTimerInit		; go to music on the timer
	call	DisplayTempBlank	; since we stop display updates, we need to blank it during play
	clrf	ARG0

MusicScoreNext:
	call	MusicNextNote	; get the next note according to the ARG0 index pointer
	movf	MUSIC_OUTER
	skpnz
	goto	MusicScoreDone	; if it is zero, the table is through, we're done

	;; otherwise, we have either a rest or a note to play

	movf	MUSIC_INNER	; if this is zero, we have a rest
	skpz
	goto	MusicScoreNote

MusicScoreRest:
	call	MusicPlayRest
	incf	ARG0
	goto	MusicScoreNext

MusicScoreNote:
	call	MusicPlayNote
	incf	ARG0
	movf	MUSIC_STACCATO
	skpz
	call	MusicPlayStaccato
	goto	MusicScoreNext

MusicScoreDone:
	call	RandomInit		; switch back to random on timer
	return


;;;
;;; MusicNextNote() - Gets the given note from the configured music table.
;;;			The music parameters are appropriately set:
;;;			MUSIC_INNER and MUSIC_OUTER as well as the staccato
;;;			and length settings.
;;;			ARG0 is the index into the current musical score.
;;;
MusicNextNote:
	movlw	0
	call	MusicTableAccess	; gets the inner value from the table
	movwf	MUSIC_INNER
	movlw	1
	call	MusicTableAccess	; gets the outer value from the table
	movwf	MUSIC_OUTER

	;; the two parameters have embeded length and staccato info that
	;; needs to be extracted

	movlw	b'11000000'	; top two bits of INNER have length
	andwf	MUSIC_INNER,W	; W now has the length
	movwf	MUSIC_LENGTH

	movlw	b'00111111'
	andwf	MUSIC_INNER	; mask off the length bits for INNER

	bcf	STATUS,C	; prepare for shifting the length
	rlf	MUSIC_LENGTH	
	rlf	MUSIC_LENGTH	
	rlf	MUSIC_LENGTH,W	; shifts through carry to get to lower bit positions
	call	MusicClickTranslate
	movwf	MUSIC_LENGTH	; now has the length in clicks

	clrf	MUSIC_STACCATO
	btfsc	MUSIC_OUTER,7
	incf	MUSIC_STACCATO

	movlw	b'01111111'
	andwf	MUSIC_OUTER	; mask off the staccato bits for OUTER
	
	return

;;;
;;; MusicTableAccess() - access the configured music table for this score.
;;;			If W is zero, the first entry is retrieved (inner) otherwise
;;;			the second entry is retrieved (outer).
;;;			ARG0 is the note entry (index) in the table (it isn't changed).
;;;
MusicTableAccess:
	movwf	SAVEW
	
	bcf	STATUS,C	; we'll shift ARG0 so we need a clear carry

	movf	MUSIC_LOC_HIGH,W
	movwf	PCLATH

	rlf	ARG0,W		; multiply ARG0 by 2 first
	movf	SAVEW
	skpz
	addlw	0x01		; if the original W was non-zero, increment to second entry

	addwf	MUSIC_LOC_LOW,W	; now add to the music location address
	skpnc
	incf	PCLATH
	movwf	PCL		; jump into configured table, which returns to this routine's caller

;;;
;;; MusicPlayNote/Rest() - assuming that the particular note has been configured,
;;;			play it for the given number of "clicks" where a
;;;			"click" is configured somewhere else.
;;;			MUSIC_LENGTH has the number of clicks, and note that it is
;;;			destroyed in the process of playing the note.
;;;
;;;			A "click" is defined as the number of cycles to cause the
;;;			TIMER0 to overflow.  This is (FOsc/4)/256/256:
;;;				1MHz	->   (/ 250000 256 256)
;;; 
MusicPlayNote:
	bsf	SOUND_PORT
	call	MusicDelayHalfWave
	bcf	SOUND_PORT
	call	MusicDelayHalfWave
	btfss	INTCON,T0IF	; if timer overflows, then we've played a click
	goto	MusicPlayNote

	bcf	INTCON,T0IF
	decfsz	MUSIC_LENGTH
	goto	MusicPlayNote

	return

MusicPlayRest:
	nop
	call	MusicDelayHalfWave
	nop
	call	MusicDelayHalfWave
	btfss	INTCON,T0IF	; if timer overflows, then we've played a click
	goto	MusicPlayRest

	bcf	INTCON,T0IF
	decfsz	MUSIC_LENGTH
	goto	MusicPlayRest

	return

;;;
;;; MusicPlayStaccato() - delays one "click" to simulate a staccato note
;;; 
MusicPlayStaccato:	
	btfss	INTCON,T0IF
	goto	MusicPlayStaccato
	bcf	INTCON,T0IF
	return
	
;;;
;;; MusicDelayHalfWave() - delays for one half-wave time
;;;				WORK0 and WORK1 used
;;; 
MusicDelayHalfWave:

	movf	MUSIC_OUTER,W
	movwf	WORK0

MPN_Outer:		
	movf	MUSIC_INNER,W
	movwf	WORK1
	
MPN_Inner:
	decfsz	WORK1
	goto	MPN_Inner

	decfsz	WORK0
	goto	MPN_Outer

	return

;;
;; MusicClick() - returns the duration in clicks based upon the value found in a note
;;		  table (only 2 bits)
;;
MusicClickTranslate:
	movwf	SAVEW
	movlw	HIGH MusicClickTable
	movwf	PCLATH
	movlw	LOW MusicClickTable
	addwf	SAVEW,W
	skpnc
	incf	PCLATH
	movwf	PCL

MusicClickTable:
	retlw	d'3'
	retlw	d'6'
	retlw	d'11'
	retlw	d'20'

;;; =======================================================================
;;; =======================================================================

	END
