;;
;; retrorover.asm
;;	Implementation of the RetroRover game.
;;
;;  In this game you are in a RetroRover and navigate your way through
;;  tougher and tougher terrain.  You score points for every mile along
;;  way.
;;
;;  Your RetroRover is a flashing dot at the bottom of the display.
;;  The terrain is every other dot that comes at you.  You need to avoid
;;  all of the terrain.
;;
;; ======================================================================
;;
;; Terrain - there are 3 types of terrain, with 2 levels in each type:
;;	Open1 - one bump every now and then
;;	Open2 - one or two bumps every now and then
;;	Cliff1 - a wall on one side of the display up to 2 pixels
;;	Cliff2 - Cliff1 plus a single bump every now and then
;;	Valley1 - walls on both sides, no less than 3 open with bumps
;;	Valley2 - walls on both sides, 1 or 2 open, no bumps (care must
;;		  be taken with Valley2 that there is an open path from
;;		  one level to the next)
;;
;; Speed - the speed of the game increases with each level of terrain
;;
;; Miles - each scroll of the display is one "mile" - there are between
;;		25 and 40 miles for each level.
;;
;; Position - the flashing RetroRover moves up one row for each type of
;;		terrain Open, Cliff, and Valley
;;
;; Scoring - each level of each terrain scores different points for each mile
;;		- Open1 - 10 points per mile
;;		- Open2 - 20 points per mile
;;		- Cliff1 - 30 points per mile
;;		- Cliff2 - 40 points per mile
;;		- Valley1 - 50 points per mile
;;		- Valley2 - 60 points per mile
;;
;;	 - one point is subtract for each movement made
;;	 - 1000 points subtracted for a crash
;;
;; Crashing - whenever a mile passes and your RetroRover occupies the same
;;		space as an obstacle - you crash
;;		- you get one crash for free, the next one kills you
;;
;; Game States -
;;	- waiting - would be nice to have a demo mode
;;	- start-up - some start-up screen
;;	- running - running the game through all terrains
;;	- crashing - if a crash occurs
;;	- end-game-crash
;;	- end-game-no-crash
;;
;; Game Variables
;;	- speed - current delay for the passing of miles - though you can
;;		  move as many times as you like before a mile goes by
;;	- lives (0 - 2 for no-lives-left through 2 lives left)
;;		(the flash is determined by this variable
;;		  0 - fast flash, 1 - flow flash, 2 - no flash)
;;	- position (0 - 2 for the RetroRover row position)
;;	- terrain - an array of 7 registers + 1 for the "stage"
;;		- gets converted and loaded to the display
;;;	- tpointer - where we are in the terrain
;;
;; Key Routines
;;	- DisplayLoad - takes the land and translates to the display
;;	- NextMileLoad - given the current mile, load the stage with the
;;			next mile according to the terrain.  Some rules
;;			exist for making sure the next mile is appropriate.
;;			Also, bumps should be inserted here.
;;	- LandScroll - scroll the land, the stage enters the top
;;			This routine does NOT check for collisions.
;;	- RoverDisplay - show the rover where it needs to be, including the
;;			flashing and position - THIS routine checks for
;;			collisions
;;	- RoverMove - checks the key strokes and moves the rover - note that
;;			you can crash into the sides of Cliffs and Valleys
;;			and this will be discovered by RoverDisplay.
;;
#include "regdef.h"
#include "portdef.h"
#include "retroroverregs.h"
#include "display.h"
#include "display.mac"
#include "button.h"
#include "button.mac"
#include "sound.h"
#include "text.h"
#include "text.mac"
#include "music.mac"
#include "utility.mac"
#include "music.h"
#include "musicdata.h"
#include "score.h"
#include "score.mac"
#include "random.h"


	global	GameInit
	global	GameProcess

;;;	global	RoverInit
;;;	global	RoverProcess

PROG1	CODE

;;;
;;; MACROS for making the code more obvious
;;;

;; note that below the 10 is hex, so it adds d'16' per mile
;; and the terrain is about 200 miles long for a best score
;; of around 3000.

ScoreAddMile:	macro
		ScoreAddMac	10
		endm

ScoreRoverMove:	macro
		ScoreSubMac	1
		endm

;; a bonus for having a rover left when you win (1k points!)

ScoreBonus:	macro
		ScoreBigAddMac	4
		endm
	


RoverGameInit:
	call	TerrainInit
	call	ScoreInit
	clrf	TSPEED
	clrf	TCURRENT
	incf	TCURRENT		; make sure it starts immediately below
	clrf	ROVER_LEVEL
	clrf	ROVER_DONE
	clrf	ROVER_STATE
	incf	ROVER_STATE		; start it off blinking slowly
	clrf	ROVER_LAST		; not REALLY correct here, but works fine
	clrf	ROVER_IMMUNITY
	movlw	d'2'
	movwf	ROVER_POS
	return

GameInit:
RoverInit:
	clrf	RR_STATE
	return

;;
;; Declare the state dispatch table for the game
;;
	StateDispatchTable	RetroR,RR_STATE,d'10'

;;
;; Here are all of the states we run in.  See RoverProcess() for a summary
;;
;;; OUT OF DATE - NEED TO REDO THIS TABLE!!!
;;; 
;;	0 - waiting for the game to start, fired up a cool little message
;;	1 - waiting for a keystroke, or the text to be done
;;		if a key, start the "ready" message and goto state 2
;;		otherwise back to state 0
;;	2 - wait for the end of the "ready" message, play charge when done
;;	3 - waiting for the message to terminate, when it does
;;		play "charge" and go to state 4
;;	4 - game is running (if win go to state 5, else state 8)
;;	5 - game has completed, flashing the display
;;	6 - game has completed, playing some victory music
;;	7 - game has completed, showing "YOU WON!" - skip to state 9
;;	8 - blew it, playing funeral march
;;	9 - show score, loop here for quite some time
;;			if a key is pressed, goto state 1
;;			otherwise eventually go to state 0

RetroR0state:

	;;
	;; the "waiting to play" state...just starts up the welcome message.
	;; not much happens in this state except starting the message
	;; it jumps right to the next state
	;;
	TextScrollInit	Welcome
	StateNext	RR_STATE
	return

RetroR1state:
	;;
	;; In this state we wait for a button to be pressed, in which case
	;; we jump to starting the game.  Otherwise we just wait for the
	;; text to be done, and we restart it by going to state 0.
	;;
	ButtonSkipIfNotPressed	0
	goto		RetroR1state2
	ButtonAck		0

	ButtonSkipIfNotPressed	1
	goto		RetroR1state2
	ButtonAck		1

	TextDoneSkip
	return
	StateSet	RR_STATE,0
	return
	
RetroR1state2:
	;;
	;; now we're ready to start the game...but first, play the charge
	;; display the "get ready message"...note that we stop the current
	;; text before moving to the next state

	TextStop
	TextFlashInit	CountDown
	StateNext	RR_STATE
	return

RetroR2state:
	;;
	;; Wait for the text to be done, play the "charge" music, reset the game
	;; settings, and jump into the game!

	TextDoneSkip
	return

	MusicScorePlay	Charge
	call		RoverGameInit
	StateNext	RR_STATE
	return

RetroR3state:
	call	RoverGameProcess
	
	movf	ROVER_DONE	; check for a winner!  ROVER_DONE will be set if so
	skpz
	goto	RetroWinner

	btfss	ROVER_STATE,2	; otherwise check for a fatal collision
	return			; just keep processing

	StateNext	RR_STATE
	return

RetroWinner:
	call		WinnerAnimationInit
	StateSet	RR_STATE,8 ; jump into the winner states!
	return
	
RetroR4state:
	;; fatal collision occurred
	
	MusicScorePlay	Dirge
	StateNext	RR_STATE
	return

RetroR5state:
	TextScrollInit	ScoreLeadin
	StateNext	RR_STATE
	return

RetroR6state:
	ButtonSkipIfNotPressed	0
	goto		RetroR6state2
	ButtonAck		0

	ButtonSkipIfNotPressed	1
	goto		RetroR6state2
	ButtonAck		1

	TextDoneSkip
	return
	call		TextScoreScrollInit
	StateNext	RR_STATE
	return

RetroR6state2:
	StateSet	RR_STATE,1
	goto		RetroR1state2

RetroR7state:
	ButtonSkipIfNotPressed	0
	goto		RetroR7state2
	ButtonAck		0

	ButtonSkipIfNotPressed	1
	goto		RetroR7state2
	ButtonAck		1

	TextDoneSkip
	return
	StateSet	RR_STATE,5
	return

RetroR7state2:
	StateSet	RR_STATE,1
	goto		RetroR1state2

RetroR8state:
	;; a winner, A WINNER!  WOW!
	;; do the initial animation
	call	WinnerAnimationProcess
	andlw	0xff			; if W is returned non-zero, we move on to the next state
	skpnz
	return				; otherwise (on zero) we just stay in this state

	btfsc	ROVER_STATE,1		; check to see if we WERE on the second rover for bonus
	goto	RetroR8state2

	;; note that the following is a macro which expands into
	;; more than one instruction, so we have to jump over it

	ScoreBonus			; used only the first rover, get a nice bonus!

RetroR8state2:
	TextScrollInit	Winner
	StateNext	RR_STATE

RetroR9state:
	;;
	;; Wait for the text to be done, then play the "winner" music
	;; and goto the "score" state

	TextDoneSkip
	return

	MusicScorePlay	Champions
	StateSet	RR_STATE,5
	return

;;
;; RoverProcess() - there are X states that the rover can be in:

GameProcess:
RoverProcess:
	StateDispatch	RetroR,RR_STATE

	
RoverGameProcess:
	call	RoverButtons

	decfsz	TCURRENT
	return

	movf	TPOINTER,W
	andlw	0x01		; get the last bit for the sound pointer
	addlw	2
	call	SoundNote
	movlw	d'128'
	call	SoundDuration

	call	TerrainSpeed
	movwf	TCURRENT
	
	movf	ROVER_IMMUNITY
	skpz
	decf	ROVER_IMMUNITY	; immunity goes away after a terrain scroll

	ScoreAddMile

	call	TerrainScroll	; upon return W has new terrain data
	
	movwf	WORK0

	btfsc	WORK0,5		; bit 5 indicates speed increase
	incf	TSPEED

	btfsc	WORK0,6		; bit 6 indicates level increase
	incf	ROVER_LEVEL

	btfsc	WORK0,7		; bit 7 indicates end of game
	incf	ROVER_DONE
	btfsc	WORK0,7		; check it again to terminate processing
	return

	call	TerrainDisplay
	goto	RoverDisplay	; paint the rover itself - and check for collisions - "tail goto"

RoverButtons:
	ButtonSkipIfNotPressed	0
	call	RoverLeft
	ButtonAck		0

	ButtonSkipIfNotPressed	1
	call	RoverRight
	ButtonAck		1

	return

RoverClearLast:
	movf	ROVER_LAST,W
	goto	ClearBit		; clear the last position - "tail goto", does this routine's return

RoverDisplay:
	movf	ROVER_STATE,W
	call	BlinkSetting		; state is verbatim blink level

	swapf	ROVER_POS,W		; get X in top bits
	iorwf	ROVER_LEVEL,W		;   and Y in lower bits

	movwf	ROVER_LAST		; save this position to clear next time

	movf	ROVER_IMMUNITY		; immunity stops collisions, although it wears
	skpz				;   off at each scroll
	goto	RoverNoCollision

	call	TerrainBump		; if zero, then the bit WAS set
	skpz
	goto	RoverNoCollision

	btfsc	ROVER_STATE,1
	goto	RoverFatal		; if already 2, then this collision was fatal

	MusicScorePlay	Warble

	movlw	ROVER_IMMUNITY_DEFAULT	; default number of scrolls of immunity
	movwf	ROVER_IMMUNITY

	incf	ROVER_STATE

RoverNoCollision:
	movf	ROVER_LAST,W
	goto	SetBit			; in any case, set the current position - "tail goto"

RoverFatal:
	bsf	ROVER_STATE,2		; indicates a fatal collision
	return

;;
;; RoverLeft() - move the Rover over one to the left
;;
RoverLeft:
	ScoreRoverMove		; each movement comes with a score penalty
	movf	ROVER_POS
	skpz			; if we are already at zero, then nothing happens
	decf	ROVER_POS
	call	RoverClearLast
	goto	RoverDisplay	; "tail goto" allows the called funtion to do the return from this one

;;
;; RoverRight() - move the Rover over one to the right
;;
RoverRight:
	ScoreRoverMove		; each movement comes with a score penalty
	movlw	ROVER_MAX
	subwf	ROVER_POS,W
	skpz			; if ROVER is already at MAX don't do anything
	incf	ROVER_POS
	call	RoverClearLast
	goto	RoverDisplay	; "tail goto"

;;;
;;; TerrainSpeed() - returns a delay that is appropriate for the
;;;			given speed setting.  Uses TSPEED.
;;;			Assumes that we have 7 possible speeds. (2 in last level)
;;; 
TerrainSpeed:
	movlw	HIGH TerrainSpeedTable
	movwf	PCLATH
	movlw	LOW TerrainSpeedTable
	addwf	TSPEED,W
	skpnc
	incf	PCLATH
	movwf	PCL
TerrainSpeedTable:
	retlw	d'250'		; speed 0
	retlw	d'230'		; speed 1
	retlw	d'210'		; speed 2
	retlw	d'190'		; speed 3
	retlw	d'170'		; speed 4
	retlw	d'150'		; speed 5
	retlw	d'130'		; speed 6

TerrainInit:
	clrf	TPOINTER
	call	TerrainClear
	return

TerrainDisplay:
	movlw	TERRAIN
	call	DisplayRotate
	return

;;;
;;; TerrainClear() - clears the terrain
;;; 
TerrainClear:
	movlw	TERRAIN_LINES
	movwf	WORK0

	movlw	TERRAIN
	movwf	FSR		; point to beginning of terrain
	
	movlw	0x1f

TCLoop:
	movwf	INDF		; clear terrain
	incf	FSR
	decfsz	WORK0
	goto	TCLoop

	return
	
;;;
;;; TerrainScroll() - scrolls the display one line.  Called whenever you want a scroll.
;;;		W has the sequence number of the terrain.
;;;		Returns the data that came into the top of the display.  The caller
;;;		should check for TERRAIN_END to determine if we are done.
;;; 
TerrainScroll:
	movf	TERRAIN_5,W
	movwf	TERRAIN_6
	
	movf	TERRAIN_4,W
	movwf	TERRAIN_5
	
	movf	TERRAIN_3,W
	movwf	TERRAIN_4
	
	movf	TERRAIN_2,W
	movwf	TERRAIN_3
	
	movf	TERRAIN_1,W
	movwf	TERRAIN_2
		
	movf	TERRAIN_0,W
	movwf	TERRAIN_1

	call	Terrain		; get the terrain to be scrolled into the top
	movwf	TERRAIN_0

	incf	TPOINTER	; point to the next terrain for next time
	return

;;
;; TerrainBump() - returns with Z set if there is a bump at (x,y).
;;		   This used to use the display memory for this, but it really
;;		   makes more sense to check the current terrain directly.
;;		   W has the (x,y) on the way in.
;;
TerrainBump:
	movwf	SAVEW
	andlw	0x0f		; get just the Y portion
	xorlw	0xff		; invert it for addition	
	addlw	TERRAIN		; add in the terrain
	addlw	d'7'		; flip up and down
	movwf	FSR		; get an index into the terrain
	swapf	SAVEW,W
	andlw	0x0f
	sublw	0x04		; flip left and right
	call	CreateMask
	andwf	INDF,W		; mask and see if target bit is set/not
				; if result is zero, then the bit WAS set
	return
	
	
;;;
;;; Terrain() - called to get the next level of terrain.  This was originally
;;;		going to be random, but this was easier, and I think it will
;;;		make better game play.
;;;
;;;		There are 256 entries in this table.
;;;
;;;		Uses TPOINTER directly
;;; 
Terrain:
	movlw	HIGH TerrainTable
	movwf	PCLATH
	movlw	LOW TerrainTable
	addwf	TPOINTER,W
	skpnc
	incf	PCLATH		; adjust if we wrapped over a 8-bit boundary
	movwf	PCL
	
;;; 
;;; There are 6 different levels in the terrain table.  The goal of the game is
;;; to last for 2 minutes if you go through the whole thing:
;;;
;;;	In this list, the lower five bits are the terrain.  The upper three
;;;	bits are:
;;;		bit 5 - next speed grade
;;;		bit 6 - next level - moves up the rover
;;;		bit 7 - end of list
	

;;; the following emacs macro expression will change all of the 1's into dots
;;; in the terrain table below...making it easier to view the terrain.

;; (let ((here (point)))	(goto-char here)	(while (re-search-forward "'\\([^']*\\)1'" nil t)	    (replace-match "'\\1.'" nil nil))	(goto-char here)	(while (re-search-forward "'\\([^']*\\)1\\(.\\)'" nil t)	    (replace-match "'\\1.\\2'" nil nil))	(goto-char here)	(while (re-search-forward "'\\([^']*\\)1\\(..\\)'" nil t)	    (replace-match "'\\1.\\2'" nil nil))	(goto-char here)	(while (re-search-forward "'\\([^']*\\)1\\(...\\)'" nil t)	    (replace-match "'\\1.\\2'" nil nil))	(goto-char here)	(while (re-search-forward "'\\([^']*\\)1\\(....\\)'" nil t)	    (replace-match "'\\1.\\2'" nil nil)))

;;; the following emacs macro expression will change it back

;;(while (re-search-forward "\\." nil t)	    (replace-match "1" nil nil))

TerrainTable:
;; Open1 (20)
	retlw	   b'11111'	; 0
	retlw	   b'11111'	; 1
	retlw	   b'11111'	; 2
	retlw	   b'11111'	; 3
	retlw	   b'10111'	; 4
	retlw	   b'11111'	; 5
	retlw	   b'11111'	; 6
	retlw	   b'11111'	; 7
	retlw	   b'10111'	; 8
	retlw	   b'11110'	; 9
	retlw	   b'11111'	; 10
	retlw	   b'11011'	; 11
	retlw	   b'11111'	; 12
	retlw	   b'10111'	; 13
	retlw	   b'11111'	; 14
	retlw	   b'11101'	; 15
	retlw	   b'01111'	; 16
	retlw	   b'11011'	; 17
	retlw	   b'11111'	; 18
	retlw	   b'11110'	; 19
;; Open2 (25)
	retlw	b'00111110'	; 20 - next speed grade
	retlw	   b'01110'	; 21
	retlw	   b'11111'	; 22
	retlw	   b'10011'	; 23
	retlw	   b'11011'	; 24
	retlw	   b'11111'	; 25
	retlw	   b'10110'	; 26
	retlw	   b'00111'	; 27
	retlw	   b'11110'	; 28
	retlw	   b'11101'	; 29
	retlw	   b'11101'	; 30
	retlw	   b'11111'	; 31
	retlw	   b'10011'	; 32
	retlw	   b'11011'	; 33
	retlw	   b'11111'	; 34
	retlw	   b'11101'	; 35
	retlw	   b'11011'	; 36
	retlw	   b'10110'	; 37
	retlw	   b'11101'	; 38
	retlw	   b'01111'	; 39
	retlw	   b'10110'	; 40
	retlw	   b'10111'	; 41
	retlw	   b'11111'	; 42
	retlw	   b'11011'	; 43
	retlw	   b'11100'	; 44
;; Cliff1 (30)
	retlw	   b'11100'	; 45
	retlw	   b'11100'	; 46
	retlw	b'01111000'	; 47 - next speed and next level
	retlw	   b'11100'	; 48
	retlw	   b'11100'	; 49
	retlw	   b'11110'	; 50
	retlw	   b'01110'	; 51
	retlw	   b'00111'	; 52
	retlw	   b'00011'	; 53
	retlw	   b'00011'	; 54
	retlw	   b'00111'	; 55
	retlw	   b'01111'	; 56
	retlw	   b'11111'	; 57
	retlw	   b'11110'	; 58
	retlw	   b'11100'	; 59
	retlw	   b'11000'	; 60
	retlw	   b'11000'	; 61
	retlw	   b'11000'	; 62
	retlw	   b'11100'	; 63
	retlw	   b'11100'	; 64
	retlw	   b'11100'	; 65
	retlw	   b'11110'	; 66
	retlw	   b'11111'	; 67
	retlw	   b'01111'	; 68
	retlw	   b'00111'	; 69
	retlw	   b'00011'	; 70
	retlw	   b'00001'	; 71
	retlw	   b'00001'	; 72
	retlw	   b'00011'	; 73
	retlw	   b'00111'	; 74
;; Cliff2 (35)
	retlw	b'00101110'	; 75 - next speed
	retlw	   b'01100'	; 76
	retlw	   b'11100'	; 77
	retlw	   b'11000'	; 78
	retlw	   b'01110'	; 79
	retlw	   b'10111'	; 80
	retlw	   b'11110'	; 81
	retlw	   b'01100'	; 82
	retlw	   b'11110'	; 83
	retlw	   b'01110'	; 84
	retlw	   b'00110'	; 85
	retlw	   b'01110'	; 86
	retlw	   b'00110'	; 87
	retlw	   b'00011'	; 88
	retlw	   b'00011'	; 89
	retlw	   b'00001'	; 90
	retlw	   b'00001'	; 91
	retlw	   b'00011'	; 92
	retlw	   b'00110'	; 93
	retlw	   b'00110'	; 94
	retlw	   b'01100'	; 95
	retlw	   b'01100'	; 96
	retlw	   b'00110'	; 97
	retlw	   b'00110'	; 98
	retlw	   b'01100'	; 99
	retlw	   b'11100'	; 100
	retlw	   b'11000'	; 101
	retlw	   b'11000'	; 102
	retlw	   b'10000'	; 103
	retlw	   b'10000'	; 104
	retlw	   b'11000'	; 105
	retlw	   b'01100'	; 106
	retlw	   b'01100'	; 107
	retlw	   b'00100'	; 108
	retlw	   b'00110'	; 109
;; Valley1 (40)
	retlw	   b'01110'	; 110
	retlw	   b'01110'	; 111
	retlw	   b'01100'	; 112
	retlw	   b'01100'	; 113
	retlw	b'01100100'	; 114 - next speed and next level
	retlw	   b'00100'	; 115
	retlw	   b'01100'	; 116
	retlw	   b'011001'	; 117
	retlw	   b'11000'	; 118
	retlw	   b'11000'	; 119
	retlw	   b'10000'	; 120
	retlw	   b'10000'	; 121
	retlw	   b'11000'	; 122
	retlw	   b'01100'	; 123
	retlw	   b'00110'	; 124
	retlw	   b'00110'	; 125
	retlw	   b'00110'	; 126
	retlw	   b'00100'	; 127
	retlw	   b'00100'	; 128
	retlw	   b'01110'	; 129
	retlw	   b'11111'	; 130
	retlw	   b'10001'	; 131
	retlw	   b'10001'	; 132
	retlw	   b'10001'	; 133 - trap occurred here - may need to punt it
	retlw	   b'10000'	; 134
	retlw	   b'11000'	; 135
	retlw	   b'11100'	; 136
	retlw	   b'01110'	; 137
	retlw	   b'00111'	; 138
	retlw	   b'00010'	; 139
	retlw	   b'00110'	; 140
	retlw	   b'01100'	; 141
	retlw	   b'01000'	; 142
	retlw	   b'01000'	; 143
	retlw	   b'11000'	; 144
	retlw	   b'01000'	; 145
	retlw	   b'11000'	; 146
	retlw	   b'10000'	; 147
	retlw	   b'11000'	; 148
	retlw	   b'01000'	; 149
;; Valley2 (40)
	retlw	b'00101000'	; 150 - next speed
	retlw	   b'01000'	; 151
	retlw	   b'01000'	; 152
	retlw	   b'01100'	; 153
	retlw	   b'00100'	; 154
	retlw	   b'00100'	; 155
	retlw	   b'01100'	; 156
	retlw	   b'01000'	; 157
	retlw	   b'01100'	; 158
	retlw	   b'00100'	; 159
	retlw	   b'00110'	; 160
	retlw	   b'00011'	; 161
	retlw	   b'00001'	; 162
	retlw	   b'00001'	; 163
	retlw	   b'00001'	; 164
	retlw	   b'00011'	; 165
	retlw	   b'00010'	; 166
	retlw	   b'00110'	; 167
	retlw	   b'01100'	; 168
	retlw	   b'11111'	; 169
	retlw	b'01111111'	; 170 - next level & speed - on level 3 now (3 below and 3 above)
	retlw	   b'11111'	; 171 - tough series here - back and forth
	retlw	   b'10000'	; 172
	retlw	   b'11111'	; 173
	retlw	   b'00100'	; 174
	retlw	   b'11111'	; 175
	retlw	   b'00001'	; 176
	retlw	   b'11111'	; 177
	retlw	   b'00001'	; 178
	retlw	   b'11111'	; 179
	retlw	   b'00100'	; 180
	retlw	   b'11111'	; 181
	retlw	   b'01110'	; 182
	retlw	   b'00100'	; 183
	retlw	   b'00100'	; 184
	retlw	   b'00100'	; 185
	retlw	   b'00100'	; 186
	retlw	   b'00100'	; 187
	retlw	   b'00100'	; 188
	retlw	   b'11111'	; 189
	retlw	   b'11111'	; 190
	retlw	   b'10101'	; 191
	retlw	   b'00100'	; 192
	retlw	   b'00000'	; 193
	retlw	   b'10001'	; 194
	retlw	   b'11111'	; 195
	retlw	b'10011111'	; 196 - stops RIGHT before it would have hit you, note that it stops
				;       BEFORE anything is painted...so it doesn't LOOK like it occurred

;;
;; WinnerAnimationProcess() - a little animation that occurs when someone wins the game.  It is based upon
;;				the state of the terrain when the game ends (the little circle).
;;		. . . . .	
;;		. o o o .	This is what is on the display after a win...the animation is 
;;		o o o o o	some simple flashing, after filling in the bottom of the circle.
;;		o o O o o
;;		. o . o .	During animation, random short sounds are generated (computer speak).
;;		. . . . .	
;;		. . . . .
;;
;;				Note that the retro registers ROVER_LAST, ROVER_POS, ROVER_DONE
;;				are usurped for this routine.

#define CHASER_TOP	ROVER_LAST
#define CHASER_BOTTOM	ROVER_POS
#define CHASER_COUNT	ROVER_DONE

;;
;; Looking for 2 full loops around the winner's ball, which is 20 steps
;;
#define WINNER_DURATION_DEFAULT	d'30'

WinnerAnimationInit:
	clrf	CHASER_TOP
	movlw	5
	movwf	CHASER_BOTTOM
	movlw	0
	call	BlinkSetting	; non-blinking
	movlw	WINNER_DURATION_DEFAULT
	movwf	CHASER_COUNT
	return

WinnerAnimationProcess:
	decfsz	TCURRENT	; the speed of the animation is equal to the speed of the last scrolling
	retlw	0		; just to make things easy

	call	TerrainSpeed
	movwf	TCURRENT

	decf	CHASER_COUNT
	skpnz
	retlw	1		; non-zero means we are done with the winner animationn

	; first set the bits of the "last" chaser location

	movf	CHASER_TOP,W
	call	Chaser
	call	SetBit
	movf	CHASER_BOTTOM,W
	call	Chaser
	call	SetBit

	; now beep!

	movf	RANDOM,W
	call	SoundNote
	movlw	d'100'
	call	SoundDuration

	; now clear the bits for the "current" chaser locations

	incf	CHASER_TOP
	movf	CHASER_TOP,W
TopReset:
	movwf	CHASER_TOP
	call	Chaser
	andlw	0xff		; need to set the Z bit if appropraite (retlw doesn't do it)
	skpnz			; if zero, we need to reset chaser index with W sitting at zero
	goto	TopReset
	call	ClearBit	; otherwise use the W that came back as the bit to clear

	incf	CHASER_BOTTOM
	movf	CHASER_BOTTOM,W
BotReset:
	movwf	CHASER_BOTTOM
	call	Chaser
	andlw	0xff		; need to set the Z bit if appropraite (retlw doesn't do it)
	skpnz			; if zero, we need to reset chaser index with W sitting at zero
	goto	BotReset
	call	ClearBit	; otherwise use the W that came back as the bit to clear

	retlw	0		; zero means we are NOT done with the winner animationn

Chaser:
	; given an index position, return the chaser for it
	; 0x00 is reserved for end of table...sure I know it is
	; a valid (x,y) value, but NOT in this context!
	; The reason this uses an end-value is to provide a way to have
	; funky chasing if we want.

	movwf	SAVEW
	movlw	HIGH ChaserTable
	movwf	PCLATH
	movlw	LOW ChaserTable
	addwf	SAVEW,W
	skpnc
	incf	PCLATH
	movwf	PCL
ChaserTable:
	retlw	0x25		; due North
	retlw	0x35
	retlw	0x44
	retlw	0x43
	retlw	0x32
	retlw	0x22		; due South
	retlw	0x12
	retlw	0x03
	retlw	0x04
	retlw	0x15
	retlw	0x00

Welcome:
	dt	"RETRO ROVER 2006 - ",0

CountDown:
	dt	"3 2 1",0
	
Winner:
	dt	" MOON BASE!",0
	
ScoreLeadin:
	dt	" --",0

;;; =======================================================================
;;; =======================================================================

	END
	