;;
;; text.asm
;;
;;    Implements text display.
;;
;;  Scrolling text is implemented by getting a column of each character
;;  and scrolling to the left.  When C1 is partly on the display, C2 is
;;  also partly on the display.
;;
;;  Each character has 4 lines.
;;
;;  The character encoding is from A=0 to Z=26, and numerals (different
;;  call) from 0 to 9.  Spaces are not encoded this way.
;;
;;; Scrolling text...
;;;
;;;   Scrolling text is implemented as a sliding window between two characters.
;;;	For this implementation, there can only be pieces of two characters
;;;	on the screen at a time.  A couple notes are important:
;;;
;;;     Characters have a single column of space between them, although each
;;;	char is only 4 columns wide.  Some chars are "proportional" in that
;;;	they are only 3 columns wide.  In this case, the code notices that
;;;	the last line was blank, and DOESN'T add the extra space.
;;;
;;;	With proportional letters, state 4 is skipped.  In other words, state
;;;	3 procedes directly to state 0.  The blank col comes from col 3 of the
;;;	char.
;;;
;;;	Right before state 0, position 0 and 1 are loaded, or reloaded as the
;;;	case may be.
;;;
;;;	Normally, a space will be given before and after a message to give it
;;;	the ability to slide in and out.
;;;
;;;            ___________________ sliding window - col 0, 1, 2, 3 of A, and a blank
;;;   0   ,---/---,
;;;       . o o . . o o o .
;;;       o . . o . o . . o
;;;       o . . o . o . . o
;;;       o o o o . o o o .
;;;       o . . o . o . . o
;;;       o . . o . o . . o
;;;       o . . o . o o o .
;;;              ___________________ sliding window - col 1, 2, 3, of A, blank, col 0 of B
;;;   1     ,---/---,
;;;       . o o . . o o o .
;;;       o . . o . o . . o
;;;       o . . o . o . . o
;;;       o o o o . o o o .
;;;       o . . o . o . . o
;;;       o . . o . o . . o
;;;       o . . o . o o o .
;;;                ___________________ sliding window - col 2, 3, of A, blank, col 0, 1 of B
;;;   2       ,---/---,
;;;       . o o . . o o o .
;;;       o . . o . o . . o
;;;       o . . o . o . . o
;;;       o o o o . o o o .
;;;       o . . o . o . . o
;;;       o . . o . o . . o
;;;       o . . o . o o o .
;;;                  ___________________ sliding window - col 3 of A, blank, col 0, 1, 2 of B
;;;   3         ,---/---,
;;;       . o o . . o o o .
;;;       o . . o . o . . o
;;;       o . . o . o . . o
;;;       o o o o . o o o .
;;;       o . . o . o . . o
;;;       o . . o . o . . o
;;;       o . . o . o o o .
;;;                    ___________________ sliding window - blank, col 0, 1, 2, 3 of B
;;;               ,---/---,			- at this point, a new character position
;;;   4   . o o . . o o o .			    can be loaded and we flip back to the
;;;       o . . o . o . . o                         first state above.
;;;       o . . o . o . . o
;;;       o o o o . o o o .
;;;       o . . o . o . . o
;;;       o . . o . o . . o
;;;       o . . o . o o o .

#include "portdef.h"
#include "regdef.h"
#include "textdata.h"
#include "text.mac"
#include "utility.mac"
#include "display.h"
#include "display.mac"
#include "score.h"

	global	TextInit
	global	TextProcess
	global	TextScrollInitCall
	global	TextFlashInitCall
	global	TextScoreFlashInit
	global	TextScoreScrollInit
	
PROG1	CODE
	
;;
;; TextInit() - initialize the scrolling text display system
;;
TextInit:
	StateSet 	TEXT_STATE,0
	movlw		TEXT_DEFAULT_SPEED
	movwf		TEXT_SPEED
	movlw		1
	movwf		TEXT_COUNTER
	movlw		0xff
	movwf		TEXT_COUNTER2
	clrf		TEXT_INDEX
	clrf		TEXT_SOURCE
	clrf		TEXT_TYPE
	return

;;
;; TextFlashInit()
;; TextScrollInit() - assuming that the message location has been loaded
;;		 in the appropriate registers, start scrolling the
;;		 text across the display.
;;
;;		 As with everything else, the Text Scrolling is state
;;		 machine based.  State includes the current location
;;		 within the text string (ordinal) and the position within
;;		 that character of the string.  A call to TextProcess
;;		 updates the display as desired.
;;
;;		 Note that the speed can be adjusted with TextSpeed().
;;
;;		 Note, too, that no display actually occurs at this point.
;;		 It is just all set-up appropriately.
;;
TextScrollInitCall:
	;; set up the appropriate state for scrolling
	;; assumes that the LOC registers are set up already
	StateSet	TEXT_STATE,1
	clrf		TEXT_INDEX
	clrf		TEXT_SOURCE	; a zero TEXT_SOURCE is simple table text
	clrf		TEXT_TYPE	; a zero TEXT_TYPE is scrolling text
	movf		TEXT_SPEED,W
	movwf		TEXT_COUNTER
	clrf		TEXT_COUNTER2
	return

TextFlashInitCall:
	call		TextScrollInitCall
	incf		TEXT_TYPE	; changes it to flashing instead of scrolling
	return
		
TextScoreScrollInit:
	call		ScoreConvert
	call		TextScrollInitCall
	incf		TEXT_SOURCE	; non-zero TEXT_SOURCE is the score
	return

TextScoreFlashInit:	
	call		ScoreConvert
	call		TextScrollInitCall
	incf		TEXT_SOURCE	; non-zero TEXT_SOURCE is the score
	incf		TEXT_TYPE	; changes it to flashing instead of scrolling
	return

;;
;; TextScoreChar() - returns the character indexed by W of the current score.
;;			Note that this routine skips leading zeros in the
;;;			score.  If total score is zero, then only one zero
;;;			is shown.  When the end of the score is reached, then
;;;			zero is returned.  This may be for different values of
;;;			W due to the lead zero skipping thing.
;;;
;;;			Note that since scores can't be otherwise configured
;;;			for a leading space, there is ALWAYS a space that
;;;			comes out ahead of the score.
;;;
;;;	NOTE - the score register is hard-coded as SCORE_BCD.
;;;
;;;	USES - WORK0
;;;
TextScoreChar:
	movwf	SAVEW
	movf	SAVEW

	skpnz
	retlw	0x20		; index zero always returns a space

	decf	SAVEW
	
	movlw	SCORE_BCD_4
	movwf	FSR

	;; first we need to go to the first non-zero digit of the score

	movlw	d'4'		; skip up to 4 leading zeros, the fifth comes out
	movwf	WORK0
TSC_Loop:	
	movf	INDF
	skpz
	goto	TSC_Continue

	decf	FSR
	decfsz	WORK0
	goto	TSC_Loop

TSC_Continue:			; now we point to the last zero or the first non-zero
				; and WORK0 indicates how many digits we have left (-1)
	movf	SAVEW,W
	subwf	WORK0,W		; if the incoming W is greater than WORK0, we are out of digits

	skpc
	retlw	0		;; if C is not set, then the result was negative (not zero or pos)
				;;	so we are out of digits to return.

	movf	SAVEW,W		; otherwise,
	subwf	FSR		;   point to the target digit - INDF now has the target char

	movlw	0x30		; offset by an ascii "0"
	addwf	INDF,W		; now W has the target character
	return
	

;;
;; TextStringChar() - get the character indexed by W from the previously
;;		configured text string.
;;
;;		You CALL this routine, which jumps to the table you
;;		previously configured, which then returns to the
;;		original caller.
;;
TextStringChar:
	movwf	WORK0		; save the index W for later

	movf	TEXT_LOC_HIGH,W
	movwf	PCLATH
	movf	TEXT_LOC_LOW,W
	addwf	WORK0,W

	skpnc			; if we got a carry, the string crosses a 256-instruction boundary
	incf	PCLATH		; in this case, update the PCLATH...and W is right from the add

	movwf	PCL		; no return or anything from THIS routine
				; it is the 'retlw' instructions in the table
				; that cause the return to the caller

;;;
;;; TextNextChar() - returns the given character in the scrolling display.
;;;			Simply a front-end that switches between text tables
;;;			and displaying a score.
;;;			W has the index of the character.
;;;
TextNextChar:
	movf	TEXT_SOURCE
	skpnz
	goto	TextStringChar
	goto	TextScoreChar

		
;;
;; Declare the state dispatch table for text
;;
	StateDispatchTable	Text,TEXT_STATE,7

;;
;; TextShortCharSet() - MACRO - sets a variable indicating if the character currently
;;				in CHAR0 is "short" - that is, it used only 3 columns.
;;				In this case the 4th column is always blank, which
;;				is how the check is done.
;;
TextShortCharSet:	macro
			movf	TEXT_CHAR1,W
			movwf	ARG0
			movlw	0x03
			call	TextASCII	; get scan col #3 for the current char into W
			movwf	TEXT_CHAR_SHORT	; save the col #3 scan col for later
			endm
			
#define BLANK_COL	0x7f

TextSkipIfNotShort:	macro
			movlw	BLANK_COL
			subwf	TEXT_CHAR_SHORT,W	; check if it is BLANK
			skpnz				; if not zero, then we DON'T have a blank
			endm

;;
;; TextProcess() - called in the normal processing loop, scroll text
;;		   across the display if it is in process.
;;;
;;;	Note that the speed is used to determine:
;;;	1 - the time between scan-line scrolls of scrolling letters
;;;	2 - the time between flashing of letters for "flash" mode
;;;	Note that the multiple for each speed is different in each mode:
;;;		scroll - 2X
;;;		flash  - 8X
;;;	That is, the time given in SPEED is multiplied that that amount
;;;	for the given mode.
;;
TextProcess:
	movf	TEXT_STATE	; cut through and return if in state zero
	skpnz
	return

	;; now execute the "standard" delay
	
	decfsz	TEXT_COUNTER	; delay for the standard speed first
	return

	movf	TEXT_SPEED,W
	movwf	TEXT_COUNTER	; ready for later processing

	;; then execute the secondary "multiplier" delay
	
	incf	TEXT_COUNTER2	; secondary counter for multiplication

	movf	TEXT_TYPE	; check for scrolling or flashing
	skpz
	goto	TextFlashProcess

TextScrollProcess:	

	btfss	TEXT_COUNTER2,0	; multiplies by two
	return

	;; now execute on a scan line scroll based upon current settings

	;; Scrolling text is done in states:
	;;  0 - not currently scrolling anything
	;;  1 - first scan line of "current" character first in view
	;;  2 - second scan line of "current" character first in view
	;;  3 - third scan line of "current" character first in view
	;;  4 - fourth scan line of "current" character first in view
	;;  5 - either the space scan is first in view, OR we jump to pseudo state 6
	;;  6 - we have ended with the "current" character and the next character
	;;	needs to be loaded as the "current" and the NEXT next needs to be loaded.

	;; now we jump into the appropriate place in the state table
	;; note that the INDEX points to the current character

TextPrepare:
	movf	TEXT_INDEX,W
	call	TextNextChar		; after this call, W has the current character
	movwf	TEXT_CHAR1		; this DOESN'T set Z (surprise to me!)
	movf	TEXT_CHAR1		; but this does

	skpnz				; if this was zero, we're done
	goto	TextDone

	TextShortCharSet	; set up to know about short columns

	incf	TEXT_INDEX,W		; otherwise go get the next char
	call	TextNextChar
	movwf	TEXT_CHAR2		; char2 has the next char or zero
	movf	TEXT_CHAR2		; another case that bit me with "movwf" not setting Z

	skpnz
	movlw	' '			; put a space in char2 if it was zero

	movwf	TEXT_CHAR2

	movf	TEXT_CHAR1,W		; load up the first character for TextASCII call later
	movwf	ARG0

	call	DisplayClear	; need to clear out blink attributes
	DisplayWindowOpen	; prepare to direct load the display

	StateDispatch		Text,TEXT_STATE


;; the above dispatch jumps directly into one of the states below, which
;; does the return necessary for the caller of TextProcess

TextDone:
	StateSet	TEXT_STATE,0
	;; fall through

Text0state:
	return				; nothing happens in state 0

Text1state:
	incf	TEXT_STATE		; prepare for state 2
	goto	ShowChar0_0

Text2state:
	incf	TEXT_STATE
	goto	ShowChar0_1

Text3state:
	incf	TEXT_STATE
	goto	ShowChar0_2

Text4state:
	TextSkipIfNotShort
	incf	TEXT_STATE		; we have a short char, move to end state
	incf	TEXT_STATE		; defaults to moving to state 4
	goto	ShowChar0_3

Text5state:
	incf	TEXT_STATE
	goto	ShowBlank

Text6state:				; a "pseudo-state" that resets to next char
	incf	TEXT_INDEX
	clrf	TEXT_STATE
	incf	TEXT_STATE
	goto	TextPrepare

;;
;; ShowChar?_X() - shows the given character (?) starting with the given position (X).
;;		   The whole character, or portion thereof, as well as the next character
;;		   are displayed.  Just enough to fill the display.  Note that all
;;		   entry points in the first char are used, and NONE in the second char.

ShowChar0_0:
	movlw	0x00		; get line 0 from char0
	call	TextASCII	; get the character scan column
	DisplayLoadColInc	; and shove it out to the display

ShowChar0_1:
	movlw	0x01		; get line 1 from char0
	call	TextASCII
	DisplayLoadColInc

ShowChar0_2:
	movlw	0x02		; get line 2 from char0
	call	TextASCII
	DisplayLoadColInc

ShowChar0_3:
	movf	TEXT_CHAR_SHORT,W	; note that we've already retrieved this scan line
	DisplayLoadColInc		; and shove it out to the display

	TextSkipIfNotShort
	goto	ShowChar1_0	; if it is short, then skip ahead

ShowBlank:
	movlw	BLANK_COL	; this is a blank
	DisplayLoadColInc	;   shove it out to the display
	DisplaySkipNotDone
	return			; if we have fully loaded the display, return

;; Entry points from here down aren't used!  But labels are kept for illustration purposes.

ShowChar1_0:
	;; load up the next character
	movf	TEXT_CHAR2,W
	movwf	ARG0

	movlw	0x00		; get line 0 from char1
	call	TextASCII	; get the character scan column
	DisplayLoadColInc	; and shove it out to the display
	DisplaySkipNotDone
	return			; if we have fully loaded the display, return

ShowChar1_1:
	movlw	0x01		; get line 1 from char1
	call	TextASCII
	DisplayLoadColInc
	DisplaySkipNotDone
	return

ShowChar1_2:
	movlw	0x02		; get line 2 from char1
	call	TextASCII
	DisplayLoadColInc
	DisplaySkipNotDone
	return

ShowChar1_3:
	movlw	0x03		; get line 3 from char0
	call	TextASCII
	DisplayLoadColInc
	return			; no choice but to return at this point


;;;
;;; TextFlashProcess() - like scrolling text, but flashing
;;; 
TextFlashProcess:	

	movlw	0x01			; adjustable multiplier, currently set to: 2
	andwf	TEXT_COUNTER2,W
	skpz
	return

	;; so the idea with flashing is that a character is placed on the display, then
	;; after a short time, another character is placed on the display.  Breaks are
	;; made by putting spaces in the string to be displayed.  

	;; there's really only one state, and each time it is effected (after the given
	;; delays) the next character is generated.

	movf	TEXT_INDEX,W
	call	TextNextChar		; after this call, W has the current character
	movwf	ARG0			; this DOESN'T set Z (surprise to me!)
	movf	ARG0			; but this does

	skpnz				; if this was zero, we're done
	goto	TextDone

	call	DisplayClear	; need to clear out blink attributes
	DisplayWindowOpen	; prepare to direct load the display

	movlw	0x7f		; first column always blank
	DisplayLoadColInc	; and shove it out to the display
	
	movlw	0x00		; get line 0
	call	TextASCII	; get the character scan column
	DisplayLoadColInc	; and shove it out to the display

	movlw	0x01		; get line 1
	call	TextASCII	; get the character scan column
	DisplayLoadColInc	; and shove it out to the display

	movlw	0x02		; get line 2
	call	TextASCII	; get the character scan column
	DisplayLoadColInc	; and shove it out to the display

	movlw	0x03		; get line 3
	call	TextASCII	; get the character scan column
	DisplayLoadColInc	; and shove it out to the display

	incf	TEXT_INDEX
	
	return

;;; =======================================================================
;;; =======================================================================

	END
	